package com.zanxiang.sdk.common.resolver;

import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.CustomException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.mybatis.entity.GameKey;
import com.zanxiang.sdk.common.annotation.ValidLogin;
import com.zanxiang.sdk.common.util.DeviceCheckUtil;
import com.zanxiang.sdk.common.util.SignUtil;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.GameKeyService;
import com.zanxiang.sdk.service.UserTokenService;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
 * token转用户信息解析器
 *
 * @author xufeng
 * @date 2022/6/20 09:59
 */
@Component
@Service
@Slf4j
public class TokenArgumentResolver implements HandlerMethodArgumentResolver {

    @Resource
    private UserTokenService userTokenService;

    @Resource
    private GameKeyService gameKeyService;

    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.getParameterType().equals(UserData.class);
    }

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest,
                                  WebDataBinderFactory binderFactory) throws Exception {
        HttpServletRequest request = webRequest.getNativeRequest(HttpServletRequest.class);
        if (request == null) {
            throw new CustomException(HttpStatusEnum.UNKNOWN_ERROR);
        }
        //签名验证
        String gameId = this.signCheck(request);
        //是否强制验证登录状态
        boolean b = parameter.hasParameterAnnotation(ValidLogin.class);
        String token = request.getHeader("token");
        //需要登录, token不存在
        if (b && StringUtils.isEmpty(token)) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        //请求头处理
        String userAgent = request.getHeader("user-agent");
        Integer deviceType = DeviceCheckUtil.getType(userAgent);
        UserData data = new UserData();
        data.setDeviceType(deviceType);
        data.setDeviceSystem(DeviceCheckUtil.getDeviceSystem(userAgent));
        data.setGameId(Long.valueOf(gameId));
        //不需要登录, token不存在
        if (StringUtils.isEmpty(token)) {
            return data;
        }
        UserTokenDTO userTokenDTO = userTokenService.getTokenInfoByTokenDevice(token, deviceType);
        if (userTokenDTO == null) {
            if (b) {
                throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
            }
            return data;
        }
        if (b && userTokenDTO.getUserId() == null) {
            throw new CustomException(HttpStatusEnum.USER_NO_LOGIN);
        }
        data.setUserId(userTokenDTO.getUserId());
        log.info("UserData:{}", data);
        return data;
    }

    private String signCheck(HttpServletRequest request) throws Exception {
        //游戏id和密钥
        String gameId = request.getHeader("gameId");
        String sign = request.getHeader("sign");
        String timestamp = request.getHeader("timestamp");
        if (Strings.isBlank(sign) || Strings.isBlank(timestamp) || Strings.isBlank(gameId)) {
            throw new CustomException(HttpStatusEnum.INVALID_PARAMS);
        }
        //签名验证
        GameKey gameKey = gameKeyService.getByGameId(Long.valueOf(gameId));
        if (gameKey == null || Strings.isBlank(gameKey.getAppKey())) {
            throw new CustomException(HttpStatusEnum.INVALID_PARAMS);
        }
        String str = "appKey=" + gameKey.getAppKey() + "&gameId=" + gameId + "&timestamp=" + timestamp;
        String mySign = SignUtil.MD5(str);
        //签名对比
        if (!Objects.equals(mySign, sign)) {
            throw new CustomException(HttpStatusEnum.INVALID_PARAMS);
        }
        return gameId;
    }
}
