package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.ExpireTimeEnum;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.text.UUID;
import com.zanxiang.common.utils.DateUtils;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.GameKey;
import com.zanxiang.mybatis.entity.UserToken;
import com.zanxiang.mybatis.mapper.UserTokenMapper;
import com.zanxiang.sdk.common.constant.RedisKeyConstant;
import com.zanxiang.sdk.common.util.RedisUtil;
import com.zanxiang.sdk.common.util.SignUtil;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.service.GameKeyService;
import com.zanxiang.sdk.service.UserTokenService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户token管理
 */
@Slf4j
@Service
public class UserTokenServiceImpl extends ServiceImpl<UserTokenMapper, UserToken> implements UserTokenService {

    @Autowired
    private RedisUtil<UserToken> redisUtil;

    @Autowired
    private GameKeyService gameKeyService;

    /**
     * 登录凭证验证
     *
     * @param gameId : 游戏id
     * @param userId : 用户id
     * @param token  : 用户token
     * @param sign   : 签名
     * @return : 返回验证结果
     */
    @Override
    public ResultVO<Long> userTokenCheck(Long gameId, Long userId, String token, String sign) {
        //查询登录签名
        GameKey gameKey = gameKeyService.getByGameId(gameId);
        if (gameKey == null || Strings.isBlank(gameKey.getLoginKey())) {
            return ResultVO.ok(-3, "游戏登录密钥不存在");
        }
        //登录密钥
        String loginKey = gameKey.getLoginKey();
        //计算用户签名
        String sb = "loginKey=" + loginKey + "&" +
                "gameId=" + gameId + "&" +
                "userId=" + userId + "&" +
                "token=" + token;
        String mySign;
        try {
            mySign = SignUtil.MD5(sb);
        } catch (Exception e) {
            throw new BaseException("MD5加密异常");
        }
        if (Objects.equals(mySign, sign)) {
            return ResultVO.ok(1, "验证成功", userId);
        }
        return ResultVO.ok(-2, "签名错误");
    }

    /**
     * 通过token设备获取token详情
     *
     * @param token      : 用户token
     * @param deviceType : 设备类型
     * @return : 返回用户token详情
     */
    @Override
    public UserTokenDTO getTokenInfoByTokenDevice(String token, Integer deviceType) {
        //查询缓存信息
        String tokenInfoKey = this.getTokenInfoKey(token, deviceType);
        UserToken userToken = redisUtil.getCache(tokenInfoKey);
        if (userToken != null) {
            return BeanUtils.copy(userToken, UserTokenDTO.class);
        }
        //缓存不存在, 查询数据库
        userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getToken, token)
                .eq(UserToken::getDeviceType, deviceType));
        if (userToken == null) {
            return null;
        }
        //设置token详情缓存
        redisUtil.setCache(tokenInfoKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //设置用户缓存
        String userTokenKey = this.getUserTokenKey(userToken.getUserId(), deviceType);
        redisUtil.setCache(userTokenKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //返回数据
        return BeanUtils.copy(userToken, UserTokenDTO.class);
    }

    /**
     * 通过用户设备获取token详情
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    @Override
    public UserTokenDTO getTokenInfoByUserDevice(Long userId, Integer deviceType) {
        //查询缓存信息
        String userTokenKey = this.getUserTokenKey(userId, deviceType);
        UserToken userToken = redisUtil.getCache(userTokenKey);
        if (userToken != null) {
            return BeanUtils.copy(userToken, UserTokenDTO.class);
        }
        //缓存不存在, 查询数据库
        userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getUserId, userId)
                .eq(UserToken::getDeviceType, deviceType));
        if (userToken == null) {
            return null;
        }
        //设置token详情缓存
        String tokenInfoKey = this.getTokenInfoKey(userToken.getToken(), deviceType);
        redisUtil.setCache(tokenInfoKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //设置用户缓存
        redisUtil.setCache(userTokenKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //返回数据
        return BeanUtils.copy(userToken, UserTokenDTO.class);
    }

    /**
     * 用户token校验
     *
     * @param token      : 用户token
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token校验结果
     */
    @Override
    public HttpStatusEnum checkUserToken(String token, Long userId, Integer deviceType) {
        UserTokenDTO userTokenDTO = this.getTokenInfoByUserDevice(userId, deviceType);
        if (userTokenDTO != null && Objects.equals(userTokenDTO.getToken(), token)) {
            return HttpStatusEnum.SUCCESS;
        }
        return HttpStatusEnum.INVALID_PARAMS;
    }

    /**
     * 获取用户token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    @Override
    public String getUserToken(Long userId, Integer deviceType) {
        //获取token
        String token = this.createToken(userId, deviceType);
        //查询数据库token信息
        UserToken userToken = super.getOne(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getUserId, userId)
                .eq(UserToken::getDeviceType, deviceType));
        //不存在token数据, 直接创建
        if (userToken == null) {
            userToken = UserToken.builder()
                    .userId(userId)
                    .token(token)
                    .deviceType(deviceType)
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .expireTime(DateUtils.localDateTimeToSecond(LocalDateTime.now()) + ExpireTimeEnum.ONE_WEEK.getTime())
                    .build();
            super.save(userToken);
        } else {
            //更新数据库
            userToken.setToken(token);
            userToken.setUpdateTime(LocalDateTime.now());
            userToken.setExpireTime(DateUtils.localDateTimeToSecond(LocalDateTime.now()) + ExpireTimeEnum.ONE_WEEK.getTime());
            super.updateById(userToken);
        }
        //设置用户缓存
        String userTokenKey = this.getUserTokenKey(userId, deviceType);
        redisUtil.setCache(userTokenKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //设置token详情缓存
        String tokenInfoKey = this.getTokenInfoKey(token, deviceType);
        redisUtil.setCache(tokenInfoKey, userToken, ExpireTimeEnum.ONE_DAY.getTime());
        //返回token
        return token;
    }

    /**
     * 删除用户token
     *
     * @param userId : 用户id
     */
    @Override
    public void deleteUserToken(Long userId) {
        List<UserToken> userTokenList = super.list(new LambdaQueryWrapper<UserToken>()
                .eq(UserToken::getUserId, userId));
        if (CollectionUtils.isEmpty(userTokenList)) {
            return;
        }
        userTokenList.forEach(userToken -> {
            //根据主键删除数据库
            super.removeById(userToken.getId());
            //删除缓存
            String userTokenKey = this.getUserTokenKey(userId, userToken.getDeviceType());
            redisUtil.deleteCache(userTokenKey);
            String tokenInfoKey = this.getTokenInfoKey(userToken.getToken(), userToken.getDeviceType());
            redisUtil.deleteCache(tokenInfoKey);
        });
    }

    /**
     * 创建token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    private String createToken(Long userId, Integer deviceType) {
        //生成随机数
        String random = userId + deviceType + UUID.randomUUID().toString() + System.currentTimeMillis();
        String charsetName = Constants.UTF8;
        //创建token
        return MD5Util.MD5Encode(MD5Util.MD5Encode(random, charsetName), charsetName);
    }

    /**
     * 拼接用户token缓存key
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回用户token的缓存key
     */
    private String getUserTokenKey(Long userId, Integer deviceType) {
        return RedisKeyConstant.USER_TOKEN_KEY + "_" + userId + "_" + deviceType;
    }

    /**
     * 拼接用户token缓存key
     *
     * @param token      : 用户token
     * @param deviceType : 设备类型
     * @return : 返回用户token的缓存key
     */
    private String getTokenInfoKey(String token, Integer deviceType) {
        return RedisKeyConstant.TOKEN_INFO_KEY + "_" + token + "_" + deviceType;
    }
}
